// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v5.29.3
// source: xds/core/v3/context_params.proto

package v3

import (
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ContextParams struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Params        map[string]string      `protobuf:"bytes,1,rep,name=params,proto3" json:"params,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ContextParams) Reset() {
	*x = ContextParams{}
	mi := &file_xds_core_v3_context_params_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ContextParams) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ContextParams) ProtoMessage() {}

func (x *ContextParams) ProtoReflect() protoreflect.Message {
	mi := &file_xds_core_v3_context_params_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ContextParams.ProtoReflect.Descriptor instead.
func (*ContextParams) Descriptor() ([]byte, []int) {
	return file_xds_core_v3_context_params_proto_rawDescGZIP(), []int{0}
}

func (x *ContextParams) GetParams() map[string]string {
	if x != nil {
		return x.Params
	}
	return nil
}

var File_xds_core_v3_context_params_proto protoreflect.FileDescriptor

const file_xds_core_v3_context_params_proto_rawDesc = "" +
	"\n" +
	" xds/core/v3/context_params.proto\x12\vxds.core.v3\x1a\x1fxds/annotations/v3/status.proto\"\x8a\x01\n" +
	"\rContextParams\x12>\n" +
	"\x06params\x18\x01 \x03(\v2&.xds.core.v3.ContextParams.ParamsEntryR\x06params\x1a9\n" +
	"\vParamsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01BZ\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"\x16com.github.xds.core.v3B\x12ContextParamsProtoP\x01Z\"github.com/cncf/xds/go/xds/core/v3b\x06proto3"

var (
	file_xds_core_v3_context_params_proto_rawDescOnce sync.Once
	file_xds_core_v3_context_params_proto_rawDescData []byte
)

func file_xds_core_v3_context_params_proto_rawDescGZIP() []byte {
	file_xds_core_v3_context_params_proto_rawDescOnce.Do(func() {
		file_xds_core_v3_context_params_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_core_v3_context_params_proto_rawDesc), len(file_xds_core_v3_context_params_proto_rawDesc)))
	})
	return file_xds_core_v3_context_params_proto_rawDescData
}

var file_xds_core_v3_context_params_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_xds_core_v3_context_params_proto_goTypes = []any{
	(*ContextParams)(nil), // 0: xds.core.v3.ContextParams
	nil,                   // 1: xds.core.v3.ContextParams.ParamsEntry
}
var file_xds_core_v3_context_params_proto_depIdxs = []int32{
	1, // 0: xds.core.v3.ContextParams.params:type_name -> xds.core.v3.ContextParams.ParamsEntry
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_xds_core_v3_context_params_proto_init() }
func file_xds_core_v3_context_params_proto_init() {
	if File_xds_core_v3_context_params_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_core_v3_context_params_proto_rawDesc), len(file_xds_core_v3_context_params_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_core_v3_context_params_proto_goTypes,
		DependencyIndexes: file_xds_core_v3_context_params_proto_depIdxs,
		MessageInfos:      file_xds_core_v3_context_params_proto_msgTypes,
	}.Build()
	File_xds_core_v3_context_params_proto = out.File
	file_xds_core_v3_context_params_proto_goTypes = nil
	file_xds_core_v3_context_params_proto_depIdxs = nil
}
