// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v5.29.3
// source: xds/type/matcher/v3/matcher.proto

package v3

import (
	v3 "github.com/cncf/xds/go/xds/core/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type Matcher struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatcherType:
	//
	//	*Matcher_MatcherList_
	//	*Matcher_MatcherTree_
	MatcherType   isMatcher_MatcherType `protobuf_oneof:"matcher_type"`
	OnNoMatch     *Matcher_OnMatch      `protobuf:"bytes,3,opt,name=on_no_match,json=onNoMatch,proto3" json:"on_no_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher) Reset() {
	*x = Matcher{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher) ProtoMessage() {}

func (x *Matcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher.ProtoReflect.Descriptor instead.
func (*Matcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0}
}

func (x *Matcher) GetMatcherType() isMatcher_MatcherType {
	if x != nil {
		return x.MatcherType
	}
	return nil
}

func (x *Matcher) GetMatcherList() *Matcher_MatcherList {
	if x != nil {
		if x, ok := x.MatcherType.(*Matcher_MatcherList_); ok {
			return x.MatcherList
		}
	}
	return nil
}

func (x *Matcher) GetMatcherTree() *Matcher_MatcherTree {
	if x != nil {
		if x, ok := x.MatcherType.(*Matcher_MatcherTree_); ok {
			return x.MatcherTree
		}
	}
	return nil
}

func (x *Matcher) GetOnNoMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnNoMatch
	}
	return nil
}

type isMatcher_MatcherType interface {
	isMatcher_MatcherType()
}

type Matcher_MatcherList_ struct {
	MatcherList *Matcher_MatcherList `protobuf:"bytes,1,opt,name=matcher_list,json=matcherList,proto3,oneof"`
}

type Matcher_MatcherTree_ struct {
	MatcherTree *Matcher_MatcherTree `protobuf:"bytes,2,opt,name=matcher_tree,json=matcherTree,proto3,oneof"`
}

func (*Matcher_MatcherList_) isMatcher_MatcherType() {}

func (*Matcher_MatcherTree_) isMatcher_MatcherType() {}

type Matcher_OnMatch struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to OnMatch:
	//
	//	*Matcher_OnMatch_Matcher
	//	*Matcher_OnMatch_Action
	OnMatch       isMatcher_OnMatch_OnMatch `protobuf_oneof:"on_match"`
	KeepMatching  bool                      `protobuf:"varint,3,opt,name=keep_matching,json=keepMatching,proto3" json:"keep_matching,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_OnMatch) Reset() {
	*x = Matcher_OnMatch{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_OnMatch) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_OnMatch) ProtoMessage() {}

func (x *Matcher_OnMatch) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_OnMatch.ProtoReflect.Descriptor instead.
func (*Matcher_OnMatch) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 0}
}

func (x *Matcher_OnMatch) GetOnMatch() isMatcher_OnMatch_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

func (x *Matcher_OnMatch) GetMatcher() *Matcher {
	if x != nil {
		if x, ok := x.OnMatch.(*Matcher_OnMatch_Matcher); ok {
			return x.Matcher
		}
	}
	return nil
}

func (x *Matcher_OnMatch) GetAction() *v3.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.OnMatch.(*Matcher_OnMatch_Action); ok {
			return x.Action
		}
	}
	return nil
}

func (x *Matcher_OnMatch) GetKeepMatching() bool {
	if x != nil {
		return x.KeepMatching
	}
	return false
}

type isMatcher_OnMatch_OnMatch interface {
	isMatcher_OnMatch_OnMatch()
}

type Matcher_OnMatch_Matcher struct {
	Matcher *Matcher `protobuf:"bytes,1,opt,name=matcher,proto3,oneof"`
}

type Matcher_OnMatch_Action struct {
	Action *v3.TypedExtensionConfig `protobuf:"bytes,2,opt,name=action,proto3,oneof"`
}

func (*Matcher_OnMatch_Matcher) isMatcher_OnMatch_OnMatch() {}

func (*Matcher_OnMatch_Action) isMatcher_OnMatch_OnMatch() {}

type Matcher_MatcherList struct {
	state         protoimpl.MessageState              `protogen:"open.v1"`
	Matchers      []*Matcher_MatcherList_FieldMatcher `protobuf:"bytes,1,rep,name=matchers,proto3" json:"matchers,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList) Reset() {
	*x = Matcher_MatcherList{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList) ProtoMessage() {}

func (x *Matcher_MatcherList) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1}
}

func (x *Matcher_MatcherList) GetMatchers() []*Matcher_MatcherList_FieldMatcher {
	if x != nil {
		return x.Matchers
	}
	return nil
}

type Matcher_MatcherTree struct {
	state protoimpl.MessageState   `protogen:"open.v1"`
	Input *v3.TypedExtensionConfig `protobuf:"bytes,1,opt,name=input,proto3" json:"input,omitempty"`
	// Types that are valid to be assigned to TreeType:
	//
	//	*Matcher_MatcherTree_ExactMatchMap
	//	*Matcher_MatcherTree_PrefixMatchMap
	//	*Matcher_MatcherTree_CustomMatch
	TreeType      isMatcher_MatcherTree_TreeType `protobuf_oneof:"tree_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherTree) Reset() {
	*x = Matcher_MatcherTree{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherTree) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherTree) ProtoMessage() {}

func (x *Matcher_MatcherTree) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherTree.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherTree) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 2}
}

func (x *Matcher_MatcherTree) GetInput() *v3.TypedExtensionConfig {
	if x != nil {
		return x.Input
	}
	return nil
}

func (x *Matcher_MatcherTree) GetTreeType() isMatcher_MatcherTree_TreeType {
	if x != nil {
		return x.TreeType
	}
	return nil
}

func (x *Matcher_MatcherTree) GetExactMatchMap() *Matcher_MatcherTree_MatchMap {
	if x != nil {
		if x, ok := x.TreeType.(*Matcher_MatcherTree_ExactMatchMap); ok {
			return x.ExactMatchMap
		}
	}
	return nil
}

func (x *Matcher_MatcherTree) GetPrefixMatchMap() *Matcher_MatcherTree_MatchMap {
	if x != nil {
		if x, ok := x.TreeType.(*Matcher_MatcherTree_PrefixMatchMap); ok {
			return x.PrefixMatchMap
		}
	}
	return nil
}

func (x *Matcher_MatcherTree) GetCustomMatch() *v3.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.TreeType.(*Matcher_MatcherTree_CustomMatch); ok {
			return x.CustomMatch
		}
	}
	return nil
}

type isMatcher_MatcherTree_TreeType interface {
	isMatcher_MatcherTree_TreeType()
}

type Matcher_MatcherTree_ExactMatchMap struct {
	ExactMatchMap *Matcher_MatcherTree_MatchMap `protobuf:"bytes,2,opt,name=exact_match_map,json=exactMatchMap,proto3,oneof"`
}

type Matcher_MatcherTree_PrefixMatchMap struct {
	PrefixMatchMap *Matcher_MatcherTree_MatchMap `protobuf:"bytes,3,opt,name=prefix_match_map,json=prefixMatchMap,proto3,oneof"`
}

type Matcher_MatcherTree_CustomMatch struct {
	CustomMatch *v3.TypedExtensionConfig `protobuf:"bytes,4,opt,name=custom_match,json=customMatch,proto3,oneof"`
}

func (*Matcher_MatcherTree_ExactMatchMap) isMatcher_MatcherTree_TreeType() {}

func (*Matcher_MatcherTree_PrefixMatchMap) isMatcher_MatcherTree_TreeType() {}

func (*Matcher_MatcherTree_CustomMatch) isMatcher_MatcherTree_TreeType() {}

type Matcher_MatcherList_Predicate struct {
	state protoimpl.MessageState `protogen:"open.v1"`
	// Types that are valid to be assigned to MatchType:
	//
	//	*Matcher_MatcherList_Predicate_SinglePredicate_
	//	*Matcher_MatcherList_Predicate_OrMatcher
	//	*Matcher_MatcherList_Predicate_AndMatcher
	//	*Matcher_MatcherList_Predicate_NotMatcher
	MatchType     isMatcher_MatcherList_Predicate_MatchType `protobuf_oneof:"match_type"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_Predicate) Reset() {
	*x = Matcher_MatcherList_Predicate{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_Predicate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_Predicate) ProtoMessage() {}

func (x *Matcher_MatcherList_Predicate) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_Predicate.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_Predicate) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 0}
}

func (x *Matcher_MatcherList_Predicate) GetMatchType() isMatcher_MatcherList_Predicate_MatchType {
	if x != nil {
		return x.MatchType
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetSinglePredicate() *Matcher_MatcherList_Predicate_SinglePredicate {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_SinglePredicate_); ok {
			return x.SinglePredicate
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetOrMatcher() *Matcher_MatcherList_Predicate_PredicateList {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_OrMatcher); ok {
			return x.OrMatcher
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetAndMatcher() *Matcher_MatcherList_Predicate_PredicateList {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_AndMatcher); ok {
			return x.AndMatcher
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate) GetNotMatcher() *Matcher_MatcherList_Predicate {
	if x != nil {
		if x, ok := x.MatchType.(*Matcher_MatcherList_Predicate_NotMatcher); ok {
			return x.NotMatcher
		}
	}
	return nil
}

type isMatcher_MatcherList_Predicate_MatchType interface {
	isMatcher_MatcherList_Predicate_MatchType()
}

type Matcher_MatcherList_Predicate_SinglePredicate_ struct {
	SinglePredicate *Matcher_MatcherList_Predicate_SinglePredicate `protobuf:"bytes,1,opt,name=single_predicate,json=singlePredicate,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_OrMatcher struct {
	OrMatcher *Matcher_MatcherList_Predicate_PredicateList `protobuf:"bytes,2,opt,name=or_matcher,json=orMatcher,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_AndMatcher struct {
	AndMatcher *Matcher_MatcherList_Predicate_PredicateList `protobuf:"bytes,3,opt,name=and_matcher,json=andMatcher,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_NotMatcher struct {
	NotMatcher *Matcher_MatcherList_Predicate `protobuf:"bytes,4,opt,name=not_matcher,json=notMatcher,proto3,oneof"`
}

func (*Matcher_MatcherList_Predicate_SinglePredicate_) isMatcher_MatcherList_Predicate_MatchType() {}

func (*Matcher_MatcherList_Predicate_OrMatcher) isMatcher_MatcherList_Predicate_MatchType() {}

func (*Matcher_MatcherList_Predicate_AndMatcher) isMatcher_MatcherList_Predicate_MatchType() {}

func (*Matcher_MatcherList_Predicate_NotMatcher) isMatcher_MatcherList_Predicate_MatchType() {}

type Matcher_MatcherList_FieldMatcher struct {
	state         protoimpl.MessageState         `protogen:"open.v1"`
	Predicate     *Matcher_MatcherList_Predicate `protobuf:"bytes,1,opt,name=predicate,proto3" json:"predicate,omitempty"`
	OnMatch       *Matcher_OnMatch               `protobuf:"bytes,2,opt,name=on_match,json=onMatch,proto3" json:"on_match,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_FieldMatcher) Reset() {
	*x = Matcher_MatcherList_FieldMatcher{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_FieldMatcher) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_FieldMatcher) ProtoMessage() {}

func (x *Matcher_MatcherList_FieldMatcher) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_FieldMatcher.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_FieldMatcher) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 1}
}

func (x *Matcher_MatcherList_FieldMatcher) GetPredicate() *Matcher_MatcherList_Predicate {
	if x != nil {
		return x.Predicate
	}
	return nil
}

func (x *Matcher_MatcherList_FieldMatcher) GetOnMatch() *Matcher_OnMatch {
	if x != nil {
		return x.OnMatch
	}
	return nil
}

type Matcher_MatcherList_Predicate_SinglePredicate struct {
	state protoimpl.MessageState   `protogen:"open.v1"`
	Input *v3.TypedExtensionConfig `protobuf:"bytes,1,opt,name=input,proto3" json:"input,omitempty"`
	// Types that are valid to be assigned to Matcher:
	//
	//	*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch
	//	*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch
	Matcher       isMatcher_MatcherList_Predicate_SinglePredicate_Matcher `protobuf_oneof:"matcher"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) Reset() {
	*x = Matcher_MatcherList_Predicate_SinglePredicate{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_Predicate_SinglePredicate) ProtoMessage() {}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_Predicate_SinglePredicate.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_Predicate_SinglePredicate) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 0, 0}
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetInput() *v3.TypedExtensionConfig {
	if x != nil {
		return x.Input
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetMatcher() isMatcher_MatcherList_Predicate_SinglePredicate_Matcher {
	if x != nil {
		return x.Matcher
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetValueMatch() *StringMatcher {
	if x != nil {
		if x, ok := x.Matcher.(*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch); ok {
			return x.ValueMatch
		}
	}
	return nil
}

func (x *Matcher_MatcherList_Predicate_SinglePredicate) GetCustomMatch() *v3.TypedExtensionConfig {
	if x != nil {
		if x, ok := x.Matcher.(*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch); ok {
			return x.CustomMatch
		}
	}
	return nil
}

type isMatcher_MatcherList_Predicate_SinglePredicate_Matcher interface {
	isMatcher_MatcherList_Predicate_SinglePredicate_Matcher()
}

type Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch struct {
	ValueMatch *StringMatcher `protobuf:"bytes,2,opt,name=value_match,json=valueMatch,proto3,oneof"`
}

type Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch struct {
	CustomMatch *v3.TypedExtensionConfig `protobuf:"bytes,3,opt,name=custom_match,json=customMatch,proto3,oneof"`
}

func (*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch) isMatcher_MatcherList_Predicate_SinglePredicate_Matcher() {
}

func (*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch) isMatcher_MatcherList_Predicate_SinglePredicate_Matcher() {
}

type Matcher_MatcherList_Predicate_PredicateList struct {
	state         protoimpl.MessageState           `protogen:"open.v1"`
	Predicate     []*Matcher_MatcherList_Predicate `protobuf:"bytes,1,rep,name=predicate,proto3" json:"predicate,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherList_Predicate_PredicateList) Reset() {
	*x = Matcher_MatcherList_Predicate_PredicateList{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[7]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherList_Predicate_PredicateList) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherList_Predicate_PredicateList) ProtoMessage() {}

func (x *Matcher_MatcherList_Predicate_PredicateList) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[7]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherList_Predicate_PredicateList.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherList_Predicate_PredicateList) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 1, 0, 1}
}

func (x *Matcher_MatcherList_Predicate_PredicateList) GetPredicate() []*Matcher_MatcherList_Predicate {
	if x != nil {
		return x.Predicate
	}
	return nil
}

type Matcher_MatcherTree_MatchMap struct {
	state         protoimpl.MessageState      `protogen:"open.v1"`
	Map           map[string]*Matcher_OnMatch `protobuf:"bytes,1,rep,name=map,proto3" json:"map,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *Matcher_MatcherTree_MatchMap) Reset() {
	*x = Matcher_MatcherTree_MatchMap{}
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[8]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Matcher_MatcherTree_MatchMap) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Matcher_MatcherTree_MatchMap) ProtoMessage() {}

func (x *Matcher_MatcherTree_MatchMap) ProtoReflect() protoreflect.Message {
	mi := &file_xds_type_matcher_v3_matcher_proto_msgTypes[8]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Matcher_MatcherTree_MatchMap.ProtoReflect.Descriptor instead.
func (*Matcher_MatcherTree_MatchMap) Descriptor() ([]byte, []int) {
	return file_xds_type_matcher_v3_matcher_proto_rawDescGZIP(), []int{0, 2, 0}
}

func (x *Matcher_MatcherTree_MatchMap) GetMap() map[string]*Matcher_OnMatch {
	if x != nil {
		return x.Map
	}
	return nil
}

var File_xds_type_matcher_v3_matcher_proto protoreflect.FileDescriptor

const file_xds_type_matcher_v3_matcher_proto_rawDesc = "" +
	"\n" +
	"!xds/type/matcher/v3/matcher.proto\x12\x13xds.type.matcher.v3\x1a\x1bxds/core/v3/extension.proto\x1a xds/type/matcher/v3/string.proto\x1a\x17validate/validate.proto\"\x9b\x10\n" +
	"\aMatcher\x12M\n" +
	"\fmatcher_list\x18\x01 \x01(\v2(.xds.type.matcher.v3.Matcher.MatcherListH\x00R\vmatcherList\x12M\n" +
	"\fmatcher_tree\x18\x02 \x01(\v2(.xds.type.matcher.v3.Matcher.MatcherTreeH\x00R\vmatcherTree\x12D\n" +
	"\von_no_match\x18\x03 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchR\tonNoMatch\x1a\xb6\x01\n" +
	"\aOnMatch\x128\n" +
	"\amatcher\x18\x01 \x01(\v2\x1c.xds.type.matcher.v3.MatcherH\x00R\amatcher\x12;\n" +
	"\x06action\x18\x02 \x01(\v2!.xds.core.v3.TypedExtensionConfigH\x00R\x06action\x12#\n" +
	"\rkeep_matching\x18\x03 \x01(\bR\fkeepMatchingB\x0f\n" +
	"\bon_match\x12\x03\xf8B\x01\x1a\xb6\b\n" +
	"\vMatcherList\x12[\n" +
	"\bmatchers\x18\x01 \x03(\v25.xds.type.matcher.v3.Matcher.MatcherList.FieldMatcherB\b\xfaB\x05\x92\x01\x02\b\x01R\bmatchers\x1a\x91\x06\n" +
	"\tPredicate\x12o\n" +
	"\x10single_predicate\x18\x01 \x01(\v2B.xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicateH\x00R\x0fsinglePredicate\x12a\n" +
	"\n" +
	"or_matcher\x18\x02 \x01(\v2@.xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateListH\x00R\torMatcher\x12c\n" +
	"\vand_matcher\x18\x03 \x01(\v2@.xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateListH\x00R\n" +
	"andMatcher\x12U\n" +
	"\vnot_matcher\x18\x04 \x01(\v22.xds.type.matcher.v3.Matcher.MatcherList.PredicateH\x00R\n" +
	"notMatcher\x1a\xf3\x01\n" +
	"\x0fSinglePredicate\x12A\n" +
	"\x05input\x18\x01 \x01(\v2!.xds.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05input\x12E\n" +
	"\vvalue_match\x18\x02 \x01(\v2\".xds.type.matcher.v3.StringMatcherH\x00R\n" +
	"valueMatch\x12F\n" +
	"\fcustom_match\x18\x03 \x01(\v2!.xds.core.v3.TypedExtensionConfigH\x00R\vcustomMatchB\x0e\n" +
	"\amatcher\x12\x03\xf8B\x01\x1ak\n" +
	"\rPredicateList\x12Z\n" +
	"\tpredicate\x18\x01 \x03(\v22.xds.type.matcher.v3.Matcher.MatcherList.PredicateB\b\xfaB\x05\x92\x01\x02\b\x02R\tpredicateB\x11\n" +
	"\n" +
	"match_type\x12\x03\xf8B\x01\x1a\xb5\x01\n" +
	"\fFieldMatcher\x12Z\n" +
	"\tpredicate\x18\x01 \x01(\v22.xds.type.matcher.v3.Matcher.MatcherList.PredicateB\b\xfaB\x05\x8a\x01\x02\x10\x01R\tpredicate\x12I\n" +
	"\bon_match\x18\x02 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchB\b\xfaB\x05\x8a\x01\x02\x10\x01R\aonMatch\x1a\xa9\x04\n" +
	"\vMatcherTree\x12A\n" +
	"\x05input\x18\x01 \x01(\v2!.xds.core.v3.TypedExtensionConfigB\b\xfaB\x05\x8a\x01\x02\x10\x01R\x05input\x12[\n" +
	"\x0fexact_match_map\x18\x02 \x01(\v21.xds.type.matcher.v3.Matcher.MatcherTree.MatchMapH\x00R\rexactMatchMap\x12]\n" +
	"\x10prefix_match_map\x18\x03 \x01(\v21.xds.type.matcher.v3.Matcher.MatcherTree.MatchMapH\x00R\x0eprefixMatchMap\x12F\n" +
	"\fcustom_match\x18\x04 \x01(\v2!.xds.core.v3.TypedExtensionConfigH\x00R\vcustomMatch\x1a\xc0\x01\n" +
	"\bMatchMap\x12V\n" +
	"\x03map\x18\x01 \x03(\v2:.xds.type.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntryB\b\xfaB\x05\x9a\x01\x02\b\x01R\x03map\x1a\\\n" +
	"\bMapEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12:\n" +
	"\x05value\x18\x02 \x01(\v2$.xds.type.matcher.v3.Matcher.OnMatchR\x05value:\x028\x01B\x10\n" +
	"\ttree_type\x12\x03\xf8B\x01B\x0e\n" +
	"\fmatcher_typeB\\\n" +
	"\x1ecom.github.xds.type.matcher.v3B\fMatcherProtoP\x01Z*github.com/cncf/xds/go/xds/type/matcher/v3b\x06proto3"

var (
	file_xds_type_matcher_v3_matcher_proto_rawDescOnce sync.Once
	file_xds_type_matcher_v3_matcher_proto_rawDescData []byte
)

func file_xds_type_matcher_v3_matcher_proto_rawDescGZIP() []byte {
	file_xds_type_matcher_v3_matcher_proto_rawDescOnce.Do(func() {
		file_xds_type_matcher_v3_matcher_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_matcher_proto_rawDesc), len(file_xds_type_matcher_v3_matcher_proto_rawDesc)))
	})
	return file_xds_type_matcher_v3_matcher_proto_rawDescData
}

var file_xds_type_matcher_v3_matcher_proto_msgTypes = make([]protoimpl.MessageInfo, 10)
var file_xds_type_matcher_v3_matcher_proto_goTypes = []any{
	(*Matcher)(nil),                                       // 0: xds.type.matcher.v3.Matcher
	(*Matcher_OnMatch)(nil),                               // 1: xds.type.matcher.v3.Matcher.OnMatch
	(*Matcher_MatcherList)(nil),                           // 2: xds.type.matcher.v3.Matcher.MatcherList
	(*Matcher_MatcherTree)(nil),                           // 3: xds.type.matcher.v3.Matcher.MatcherTree
	(*Matcher_MatcherList_Predicate)(nil),                 // 4: xds.type.matcher.v3.Matcher.MatcherList.Predicate
	(*Matcher_MatcherList_FieldMatcher)(nil),              // 5: xds.type.matcher.v3.Matcher.MatcherList.FieldMatcher
	(*Matcher_MatcherList_Predicate_SinglePredicate)(nil), // 6: xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate
	(*Matcher_MatcherList_Predicate_PredicateList)(nil),   // 7: xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateList
	(*Matcher_MatcherTree_MatchMap)(nil),                  // 8: xds.type.matcher.v3.Matcher.MatcherTree.MatchMap
	nil,                                                   // 9: xds.type.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry
	(*v3.TypedExtensionConfig)(nil),                       // 10: xds.core.v3.TypedExtensionConfig
	(*StringMatcher)(nil),                                 // 11: xds.type.matcher.v3.StringMatcher
}
var file_xds_type_matcher_v3_matcher_proto_depIdxs = []int32{
	2,  // 0: xds.type.matcher.v3.Matcher.matcher_list:type_name -> xds.type.matcher.v3.Matcher.MatcherList
	3,  // 1: xds.type.matcher.v3.Matcher.matcher_tree:type_name -> xds.type.matcher.v3.Matcher.MatcherTree
	1,  // 2: xds.type.matcher.v3.Matcher.on_no_match:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	0,  // 3: xds.type.matcher.v3.Matcher.OnMatch.matcher:type_name -> xds.type.matcher.v3.Matcher
	10, // 4: xds.type.matcher.v3.Matcher.OnMatch.action:type_name -> xds.core.v3.TypedExtensionConfig
	5,  // 5: xds.type.matcher.v3.Matcher.MatcherList.matchers:type_name -> xds.type.matcher.v3.Matcher.MatcherList.FieldMatcher
	10, // 6: xds.type.matcher.v3.Matcher.MatcherTree.input:type_name -> xds.core.v3.TypedExtensionConfig
	8,  // 7: xds.type.matcher.v3.Matcher.MatcherTree.exact_match_map:type_name -> xds.type.matcher.v3.Matcher.MatcherTree.MatchMap
	8,  // 8: xds.type.matcher.v3.Matcher.MatcherTree.prefix_match_map:type_name -> xds.type.matcher.v3.Matcher.MatcherTree.MatchMap
	10, // 9: xds.type.matcher.v3.Matcher.MatcherTree.custom_match:type_name -> xds.core.v3.TypedExtensionConfig
	6,  // 10: xds.type.matcher.v3.Matcher.MatcherList.Predicate.single_predicate:type_name -> xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate
	7,  // 11: xds.type.matcher.v3.Matcher.MatcherList.Predicate.or_matcher:type_name -> xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateList
	7,  // 12: xds.type.matcher.v3.Matcher.MatcherList.Predicate.and_matcher:type_name -> xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateList
	4,  // 13: xds.type.matcher.v3.Matcher.MatcherList.Predicate.not_matcher:type_name -> xds.type.matcher.v3.Matcher.MatcherList.Predicate
	4,  // 14: xds.type.matcher.v3.Matcher.MatcherList.FieldMatcher.predicate:type_name -> xds.type.matcher.v3.Matcher.MatcherList.Predicate
	1,  // 15: xds.type.matcher.v3.Matcher.MatcherList.FieldMatcher.on_match:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	10, // 16: xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate.input:type_name -> xds.core.v3.TypedExtensionConfig
	11, // 17: xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate.value_match:type_name -> xds.type.matcher.v3.StringMatcher
	10, // 18: xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate.custom_match:type_name -> xds.core.v3.TypedExtensionConfig
	4,  // 19: xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateList.predicate:type_name -> xds.type.matcher.v3.Matcher.MatcherList.Predicate
	9,  // 20: xds.type.matcher.v3.Matcher.MatcherTree.MatchMap.map:type_name -> xds.type.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry
	1,  // 21: xds.type.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry.value:type_name -> xds.type.matcher.v3.Matcher.OnMatch
	22, // [22:22] is the sub-list for method output_type
	22, // [22:22] is the sub-list for method input_type
	22, // [22:22] is the sub-list for extension type_name
	22, // [22:22] is the sub-list for extension extendee
	0,  // [0:22] is the sub-list for field type_name
}

func init() { file_xds_type_matcher_v3_matcher_proto_init() }
func file_xds_type_matcher_v3_matcher_proto_init() {
	if File_xds_type_matcher_v3_matcher_proto != nil {
		return
	}
	file_xds_type_matcher_v3_string_proto_init()
	file_xds_type_matcher_v3_matcher_proto_msgTypes[0].OneofWrappers = []any{
		(*Matcher_MatcherList_)(nil),
		(*Matcher_MatcherTree_)(nil),
	}
	file_xds_type_matcher_v3_matcher_proto_msgTypes[1].OneofWrappers = []any{
		(*Matcher_OnMatch_Matcher)(nil),
		(*Matcher_OnMatch_Action)(nil),
	}
	file_xds_type_matcher_v3_matcher_proto_msgTypes[3].OneofWrappers = []any{
		(*Matcher_MatcherTree_ExactMatchMap)(nil),
		(*Matcher_MatcherTree_PrefixMatchMap)(nil),
		(*Matcher_MatcherTree_CustomMatch)(nil),
	}
	file_xds_type_matcher_v3_matcher_proto_msgTypes[4].OneofWrappers = []any{
		(*Matcher_MatcherList_Predicate_SinglePredicate_)(nil),
		(*Matcher_MatcherList_Predicate_OrMatcher)(nil),
		(*Matcher_MatcherList_Predicate_AndMatcher)(nil),
		(*Matcher_MatcherList_Predicate_NotMatcher)(nil),
	}
	file_xds_type_matcher_v3_matcher_proto_msgTypes[6].OneofWrappers = []any{
		(*Matcher_MatcherList_Predicate_SinglePredicate_ValueMatch)(nil),
		(*Matcher_MatcherList_Predicate_SinglePredicate_CustomMatch)(nil),
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_type_matcher_v3_matcher_proto_rawDesc), len(file_xds_type_matcher_v3_matcher_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   10,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_type_matcher_v3_matcher_proto_goTypes,
		DependencyIndexes: file_xds_type_matcher_v3_matcher_proto_depIdxs,
		MessageInfos:      file_xds_type_matcher_v3_matcher_proto_msgTypes,
	}.Build()
	File_xds_type_matcher_v3_matcher_proto = out.File
	file_xds_type_matcher_v3_matcher_proto_goTypes = nil
	file_xds_type_matcher_v3_matcher_proto_depIdxs = nil
}
