// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.10
// 	protoc        v5.29.3
// source: xds/core/v3/resource_name.proto

package v3

import (
	_ "github.com/cncf/xds/go/xds/annotations/v3"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type ResourceName struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            string                 `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	Authority     string                 `protobuf:"bytes,2,opt,name=authority,proto3" json:"authority,omitempty"`
	ResourceType  string                 `protobuf:"bytes,3,opt,name=resource_type,json=resourceType,proto3" json:"resource_type,omitempty"`
	Context       *ContextParams         `protobuf:"bytes,4,opt,name=context,proto3" json:"context,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *ResourceName) Reset() {
	*x = ResourceName{}
	mi := &file_xds_core_v3_resource_name_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *ResourceName) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*ResourceName) ProtoMessage() {}

func (x *ResourceName) ProtoReflect() protoreflect.Message {
	mi := &file_xds_core_v3_resource_name_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use ResourceName.ProtoReflect.Descriptor instead.
func (*ResourceName) Descriptor() ([]byte, []int) {
	return file_xds_core_v3_resource_name_proto_rawDescGZIP(), []int{0}
}

func (x *ResourceName) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

func (x *ResourceName) GetAuthority() string {
	if x != nil {
		return x.Authority
	}
	return ""
}

func (x *ResourceName) GetResourceType() string {
	if x != nil {
		return x.ResourceType
	}
	return ""
}

func (x *ResourceName) GetContext() *ContextParams {
	if x != nil {
		return x.Context
	}
	return nil
}

var File_xds_core_v3_resource_name_proto protoreflect.FileDescriptor

const file_xds_core_v3_resource_name_proto_rawDesc = "" +
	"\n" +
	"\x1fxds/core/v3/resource_name.proto\x12\vxds.core.v3\x1a\x1fxds/annotations/v3/status.proto\x1a xds/core/v3/context_params.proto\x1a\x17validate/validate.proto\"\xa0\x01\n" +
	"\fResourceName\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n" +
	"\tauthority\x18\x02 \x01(\tR\tauthority\x12,\n" +
	"\rresource_type\x18\x03 \x01(\tB\a\xfaB\x04r\x02\x10\x01R\fresourceType\x124\n" +
	"\acontext\x18\x04 \x01(\v2\x1a.xds.core.v3.ContextParamsR\acontextBY\xd2Ƥ\xe1\x06\x02\b\x01\n" +
	"\x16com.github.xds.core.v3B\x11ResourceNameProtoP\x01Z\"github.com/cncf/xds/go/xds/core/v3b\x06proto3"

var (
	file_xds_core_v3_resource_name_proto_rawDescOnce sync.Once
	file_xds_core_v3_resource_name_proto_rawDescData []byte
)

func file_xds_core_v3_resource_name_proto_rawDescGZIP() []byte {
	file_xds_core_v3_resource_name_proto_rawDescOnce.Do(func() {
		file_xds_core_v3_resource_name_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_xds_core_v3_resource_name_proto_rawDesc), len(file_xds_core_v3_resource_name_proto_rawDesc)))
	})
	return file_xds_core_v3_resource_name_proto_rawDescData
}

var file_xds_core_v3_resource_name_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_xds_core_v3_resource_name_proto_goTypes = []any{
	(*ResourceName)(nil),  // 0: xds.core.v3.ResourceName
	(*ContextParams)(nil), // 1: xds.core.v3.ContextParams
}
var file_xds_core_v3_resource_name_proto_depIdxs = []int32{
	1, // 0: xds.core.v3.ResourceName.context:type_name -> xds.core.v3.ContextParams
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_xds_core_v3_resource_name_proto_init() }
func file_xds_core_v3_resource_name_proto_init() {
	if File_xds_core_v3_resource_name_proto != nil {
		return
	}
	file_xds_core_v3_context_params_proto_init()
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_xds_core_v3_resource_name_proto_rawDesc), len(file_xds_core_v3_resource_name_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_xds_core_v3_resource_name_proto_goTypes,
		DependencyIndexes: file_xds_core_v3_resource_name_proto_depIdxs,
		MessageInfos:      file_xds_core_v3_resource_name_proto_msgTypes,
	}.Build()
	File_xds_core_v3_resource_name_proto = out.File
	file_xds_core_v3_resource_name_proto_goTypes = nil
	file_xds_core_v3_resource_name_proto_depIdxs = nil
}
