package io

const (
	// Byte is 8 bits
	Byte int64 = 1
	// KibiByte (KiB) is 1024 Bytes
	KibiByte = Byte * 1024
	// MebiByte (MiB) is 1024 KiB
	MebiByte = KibiByte * 1024
	// GibiByte (GiB) is 1024 MiB
	GibiByte = MebiByte * 1024
)
