// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.33.0
// 	protoc        v4.25.2
// source: restoreitemaction/v2/RestoreItemAction.proto

package v2

import (
	generated "github.com/vmware-tanzu/velero/pkg/plugin/generated"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	emptypb "google.golang.org/protobuf/types/known/emptypb"
	reflect "reflect"
	sync "sync"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type RestoreItemActionExecuteRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Plugin         string `protobuf:"bytes,1,opt,name=plugin,proto3" json:"plugin,omitempty"`
	Item           []byte `protobuf:"bytes,2,opt,name=item,proto3" json:"item,omitempty"`
	Restore        []byte `protobuf:"bytes,3,opt,name=restore,proto3" json:"restore,omitempty"`
	ItemFromBackup []byte `protobuf:"bytes,4,opt,name=itemFromBackup,proto3" json:"itemFromBackup,omitempty"`
}

func (x *RestoreItemActionExecuteRequest) Reset() {
	*x = RestoreItemActionExecuteRequest{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[0]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionExecuteRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionExecuteRequest) ProtoMessage() {}

func (x *RestoreItemActionExecuteRequest) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[0]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionExecuteRequest.ProtoReflect.Descriptor instead.
func (*RestoreItemActionExecuteRequest) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{0}
}

func (x *RestoreItemActionExecuteRequest) GetPlugin() string {
	if x != nil {
		return x.Plugin
	}
	return ""
}

func (x *RestoreItemActionExecuteRequest) GetItem() []byte {
	if x != nil {
		return x.Item
	}
	return nil
}

func (x *RestoreItemActionExecuteRequest) GetRestore() []byte {
	if x != nil {
		return x.Restore
	}
	return nil
}

func (x *RestoreItemActionExecuteRequest) GetItemFromBackup() []byte {
	if x != nil {
		return x.ItemFromBackup
	}
	return nil
}

type RestoreItemActionExecuteResponse struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Item                        []byte                          `protobuf:"bytes,1,opt,name=item,proto3" json:"item,omitempty"`
	AdditionalItems             []*generated.ResourceIdentifier `protobuf:"bytes,2,rep,name=additionalItems,proto3" json:"additionalItems,omitempty"`
	SkipRestore                 bool                            `protobuf:"varint,3,opt,name=skipRestore,proto3" json:"skipRestore,omitempty"`
	OperationID                 string                          `protobuf:"bytes,4,opt,name=operationID,proto3" json:"operationID,omitempty"`
	WaitForAdditionalItems      bool                            `protobuf:"varint,5,opt,name=waitForAdditionalItems,proto3" json:"waitForAdditionalItems,omitempty"`
	AdditionalItemsReadyTimeout *durationpb.Duration            `protobuf:"bytes,6,opt,name=additionalItemsReadyTimeout,proto3" json:"additionalItemsReadyTimeout,omitempty"`
}

func (x *RestoreItemActionExecuteResponse) Reset() {
	*x = RestoreItemActionExecuteResponse{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[1]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionExecuteResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionExecuteResponse) ProtoMessage() {}

func (x *RestoreItemActionExecuteResponse) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[1]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionExecuteResponse.ProtoReflect.Descriptor instead.
func (*RestoreItemActionExecuteResponse) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{1}
}

func (x *RestoreItemActionExecuteResponse) GetItem() []byte {
	if x != nil {
		return x.Item
	}
	return nil
}

func (x *RestoreItemActionExecuteResponse) GetAdditionalItems() []*generated.ResourceIdentifier {
	if x != nil {
		return x.AdditionalItems
	}
	return nil
}

func (x *RestoreItemActionExecuteResponse) GetSkipRestore() bool {
	if x != nil {
		return x.SkipRestore
	}
	return false
}

func (x *RestoreItemActionExecuteResponse) GetOperationID() string {
	if x != nil {
		return x.OperationID
	}
	return ""
}

func (x *RestoreItemActionExecuteResponse) GetWaitForAdditionalItems() bool {
	if x != nil {
		return x.WaitForAdditionalItems
	}
	return false
}

func (x *RestoreItemActionExecuteResponse) GetAdditionalItemsReadyTimeout() *durationpb.Duration {
	if x != nil {
		return x.AdditionalItemsReadyTimeout
	}
	return nil
}

type RestoreItemActionAppliesToRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Plugin string `protobuf:"bytes,1,opt,name=plugin,proto3" json:"plugin,omitempty"`
}

func (x *RestoreItemActionAppliesToRequest) Reset() {
	*x = RestoreItemActionAppliesToRequest{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[2]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionAppliesToRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionAppliesToRequest) ProtoMessage() {}

func (x *RestoreItemActionAppliesToRequest) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[2]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionAppliesToRequest.ProtoReflect.Descriptor instead.
func (*RestoreItemActionAppliesToRequest) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{2}
}

func (x *RestoreItemActionAppliesToRequest) GetPlugin() string {
	if x != nil {
		return x.Plugin
	}
	return ""
}

type RestoreItemActionAppliesToResponse struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	ResourceSelector *generated.ResourceSelector `protobuf:"bytes,1,opt,name=ResourceSelector,proto3" json:"ResourceSelector,omitempty"`
}

func (x *RestoreItemActionAppliesToResponse) Reset() {
	*x = RestoreItemActionAppliesToResponse{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[3]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionAppliesToResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionAppliesToResponse) ProtoMessage() {}

func (x *RestoreItemActionAppliesToResponse) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[3]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionAppliesToResponse.ProtoReflect.Descriptor instead.
func (*RestoreItemActionAppliesToResponse) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{3}
}

func (x *RestoreItemActionAppliesToResponse) GetResourceSelector() *generated.ResourceSelector {
	if x != nil {
		return x.ResourceSelector
	}
	return nil
}

type RestoreItemActionProgressRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Plugin      string `protobuf:"bytes,1,opt,name=plugin,proto3" json:"plugin,omitempty"`
	OperationID string `protobuf:"bytes,2,opt,name=operationID,proto3" json:"operationID,omitempty"`
	Restore     []byte `protobuf:"bytes,3,opt,name=restore,proto3" json:"restore,omitempty"`
}

func (x *RestoreItemActionProgressRequest) Reset() {
	*x = RestoreItemActionProgressRequest{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[4]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionProgressRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionProgressRequest) ProtoMessage() {}

func (x *RestoreItemActionProgressRequest) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[4]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionProgressRequest.ProtoReflect.Descriptor instead.
func (*RestoreItemActionProgressRequest) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{4}
}

func (x *RestoreItemActionProgressRequest) GetPlugin() string {
	if x != nil {
		return x.Plugin
	}
	return ""
}

func (x *RestoreItemActionProgressRequest) GetOperationID() string {
	if x != nil {
		return x.OperationID
	}
	return ""
}

func (x *RestoreItemActionProgressRequest) GetRestore() []byte {
	if x != nil {
		return x.Restore
	}
	return nil
}

type RestoreItemActionProgressResponse struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Progress *generated.OperationProgress `protobuf:"bytes,1,opt,name=progress,proto3" json:"progress,omitempty"`
}

func (x *RestoreItemActionProgressResponse) Reset() {
	*x = RestoreItemActionProgressResponse{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[5]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionProgressResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionProgressResponse) ProtoMessage() {}

func (x *RestoreItemActionProgressResponse) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[5]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionProgressResponse.ProtoReflect.Descriptor instead.
func (*RestoreItemActionProgressResponse) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{5}
}

func (x *RestoreItemActionProgressResponse) GetProgress() *generated.OperationProgress {
	if x != nil {
		return x.Progress
	}
	return nil
}

type RestoreItemActionCancelRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Plugin      string `protobuf:"bytes,1,opt,name=plugin,proto3" json:"plugin,omitempty"`
	OperationID string `protobuf:"bytes,2,opt,name=operationID,proto3" json:"operationID,omitempty"`
	Restore     []byte `protobuf:"bytes,3,opt,name=restore,proto3" json:"restore,omitempty"`
}

func (x *RestoreItemActionCancelRequest) Reset() {
	*x = RestoreItemActionCancelRequest{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[6]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionCancelRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionCancelRequest) ProtoMessage() {}

func (x *RestoreItemActionCancelRequest) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[6]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionCancelRequest.ProtoReflect.Descriptor instead.
func (*RestoreItemActionCancelRequest) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{6}
}

func (x *RestoreItemActionCancelRequest) GetPlugin() string {
	if x != nil {
		return x.Plugin
	}
	return ""
}

func (x *RestoreItemActionCancelRequest) GetOperationID() string {
	if x != nil {
		return x.OperationID
	}
	return ""
}

func (x *RestoreItemActionCancelRequest) GetRestore() []byte {
	if x != nil {
		return x.Restore
	}
	return nil
}

type RestoreItemActionItemsReadyRequest struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Plugin          string                          `protobuf:"bytes,1,opt,name=plugin,proto3" json:"plugin,omitempty"`
	Restore         []byte                          `protobuf:"bytes,2,opt,name=restore,proto3" json:"restore,omitempty"`
	AdditionalItems []*generated.ResourceIdentifier `protobuf:"bytes,3,rep,name=additionalItems,proto3" json:"additionalItems,omitempty"`
}

func (x *RestoreItemActionItemsReadyRequest) Reset() {
	*x = RestoreItemActionItemsReadyRequest{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[7]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionItemsReadyRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionItemsReadyRequest) ProtoMessage() {}

func (x *RestoreItemActionItemsReadyRequest) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[7]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionItemsReadyRequest.ProtoReflect.Descriptor instead.
func (*RestoreItemActionItemsReadyRequest) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{7}
}

func (x *RestoreItemActionItemsReadyRequest) GetPlugin() string {
	if x != nil {
		return x.Plugin
	}
	return ""
}

func (x *RestoreItemActionItemsReadyRequest) GetRestore() []byte {
	if x != nil {
		return x.Restore
	}
	return nil
}

func (x *RestoreItemActionItemsReadyRequest) GetAdditionalItems() []*generated.ResourceIdentifier {
	if x != nil {
		return x.AdditionalItems
	}
	return nil
}

type RestoreItemActionItemsReadyResponse struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	Ready bool `protobuf:"varint,1,opt,name=ready,proto3" json:"ready,omitempty"`
}

func (x *RestoreItemActionItemsReadyResponse) Reset() {
	*x = RestoreItemActionItemsReadyResponse{}
	if protoimpl.UnsafeEnabled {
		mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[8]
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		ms.StoreMessageInfo(mi)
	}
}

func (x *RestoreItemActionItemsReadyResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*RestoreItemActionItemsReadyResponse) ProtoMessage() {}

func (x *RestoreItemActionItemsReadyResponse) ProtoReflect() protoreflect.Message {
	mi := &file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[8]
	if protoimpl.UnsafeEnabled && x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use RestoreItemActionItemsReadyResponse.ProtoReflect.Descriptor instead.
func (*RestoreItemActionItemsReadyResponse) Descriptor() ([]byte, []int) {
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP(), []int{8}
}

func (x *RestoreItemActionItemsReadyResponse) GetReady() bool {
	if x != nil {
		return x.Ready
	}
	return false
}

var File_restoreitemaction_v2_RestoreItemAction_proto protoreflect.FileDescriptor

var file_restoreitemaction_v2_RestoreItemAction_proto_rawDesc = []byte{
	0x0a, 0x2c, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x69, 0x74, 0x65, 0x6d, 0x61, 0x63, 0x74,
	0x69, 0x6f, 0x6e, 0x2f, 0x76, 0x32, 0x2f, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74,
	0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x12, 0x02,
	0x76, 0x32, 0x1a, 0x0c, 0x53, 0x68, 0x61, 0x72, 0x65, 0x64, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f,
	0x1a, 0x1b, 0x67, 0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2f, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x62, 0x75,
	0x66, 0x2f, 0x65, 0x6d, 0x70, 0x74, 0x79, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x1a, 0x1e, 0x67,
	0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2f, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x62, 0x75, 0x66, 0x2f, 0x64,
	0x75, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x22, 0x8f, 0x01,
	0x0a, 0x1f, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74,
	0x69, 0x6f, 0x6e, 0x45, 0x78, 0x65, 0x63, 0x75, 0x74, 0x65, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73,
	0x74, 0x12, 0x16, 0x0a, 0x06, 0x70, 0x6c, 0x75, 0x67, 0x69, 0x6e, 0x18, 0x01, 0x20, 0x01, 0x28,
	0x09, 0x52, 0x06, 0x70, 0x6c, 0x75, 0x67, 0x69, 0x6e, 0x12, 0x12, 0x0a, 0x04, 0x69, 0x74, 0x65,
	0x6d, 0x18, 0x02, 0x20, 0x01, 0x28, 0x0c, 0x52, 0x04, 0x69, 0x74, 0x65, 0x6d, 0x12, 0x18, 0x0a,
	0x07, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x18, 0x03, 0x20, 0x01, 0x28, 0x0c, 0x52, 0x07,
	0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x12, 0x26, 0x0a, 0x0e, 0x69, 0x74, 0x65, 0x6d, 0x46,
	0x72, 0x6f, 0x6d, 0x42, 0x61, 0x63, 0x6b, 0x75, 0x70, 0x18, 0x04, 0x20, 0x01, 0x28, 0x0c, 0x52,
	0x0e, 0x69, 0x74, 0x65, 0x6d, 0x46, 0x72, 0x6f, 0x6d, 0x42, 0x61, 0x63, 0x6b, 0x75, 0x70, 0x22,
	0xd8, 0x02, 0x0a, 0x20, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41,
	0x63, 0x74, 0x69, 0x6f, 0x6e, 0x45, 0x78, 0x65, 0x63, 0x75, 0x74, 0x65, 0x52, 0x65, 0x73, 0x70,
	0x6f, 0x6e, 0x73, 0x65, 0x12, 0x12, 0x0a, 0x04, 0x69, 0x74, 0x65, 0x6d, 0x18, 0x01, 0x20, 0x01,
	0x28, 0x0c, 0x52, 0x04, 0x69, 0x74, 0x65, 0x6d, 0x12, 0x47, 0x0a, 0x0f, 0x61, 0x64, 0x64, 0x69,
	0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x18, 0x02, 0x20, 0x03, 0x28,
	0x0b, 0x32, 0x1d, 0x2e, 0x67, 0x65, 0x6e, 0x65, 0x72, 0x61, 0x74, 0x65, 0x64, 0x2e, 0x52, 0x65,
	0x73, 0x6f, 0x75, 0x72, 0x63, 0x65, 0x49, 0x64, 0x65, 0x6e, 0x74, 0x69, 0x66, 0x69, 0x65, 0x72,
	0x52, 0x0f, 0x61, 0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d,
	0x73, 0x12, 0x20, 0x0a, 0x0b, 0x73, 0x6b, 0x69, 0x70, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65,
	0x18, 0x03, 0x20, 0x01, 0x28, 0x08, 0x52, 0x0b, 0x73, 0x6b, 0x69, 0x70, 0x52, 0x65, 0x73, 0x74,
	0x6f, 0x72, 0x65, 0x12, 0x20, 0x0a, 0x0b, 0x6f, 0x70, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e,
	0x49, 0x44, 0x18, 0x04, 0x20, 0x01, 0x28, 0x09, 0x52, 0x0b, 0x6f, 0x70, 0x65, 0x72, 0x61, 0x74,
	0x69, 0x6f, 0x6e, 0x49, 0x44, 0x12, 0x36, 0x0a, 0x16, 0x77, 0x61, 0x69, 0x74, 0x46, 0x6f, 0x72,
	0x41, 0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x18,
	0x05, 0x20, 0x01, 0x28, 0x08, 0x52, 0x16, 0x77, 0x61, 0x69, 0x74, 0x46, 0x6f, 0x72, 0x41, 0x64,
	0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x12, 0x5b, 0x0a,
	0x1b, 0x61, 0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73,
	0x52, 0x65, 0x61, 0x64, 0x79, 0x54, 0x69, 0x6d, 0x65, 0x6f, 0x75, 0x74, 0x18, 0x06, 0x20, 0x01,
	0x28, 0x0b, 0x32, 0x19, 0x2e, 0x67, 0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2e, 0x70, 0x72, 0x6f, 0x74,
	0x6f, 0x62, 0x75, 0x66, 0x2e, 0x44, 0x75, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x52, 0x1b, 0x61,
	0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x52, 0x65,
	0x61, 0x64, 0x79, 0x54, 0x69, 0x6d, 0x65, 0x6f, 0x75, 0x74, 0x22, 0x3b, 0x0a, 0x21, 0x52, 0x65,
	0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x41,
	0x70, 0x70, 0x6c, 0x69, 0x65, 0x73, 0x54, 0x6f, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x12,
	0x16, 0x0a, 0x06, 0x70, 0x6c, 0x75, 0x67, 0x69, 0x6e, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52,
	0x06, 0x70, 0x6c, 0x75, 0x67, 0x69, 0x6e, 0x22, 0x6d, 0x0a, 0x22, 0x52, 0x65, 0x73, 0x74, 0x6f,
	0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x41, 0x70, 0x70, 0x6c,
	0x69, 0x65, 0x73, 0x54, 0x6f, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x47, 0x0a,
	0x10, 0x52, 0x65, 0x73, 0x6f, 0x75, 0x72, 0x63, 0x65, 0x53, 0x65, 0x6c, 0x65, 0x63, 0x74, 0x6f,
	0x72, 0x18, 0x01, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x1b, 0x2e, 0x67, 0x65, 0x6e, 0x65, 0x72, 0x61,
	0x74, 0x65, 0x64, 0x2e, 0x52, 0x65, 0x73, 0x6f, 0x75, 0x72, 0x63, 0x65, 0x53, 0x65, 0x6c, 0x65,
	0x63, 0x74, 0x6f, 0x72, 0x52, 0x10, 0x52, 0x65, 0x73, 0x6f, 0x75, 0x72, 0x63, 0x65, 0x53, 0x65,
	0x6c, 0x65, 0x63, 0x74, 0x6f, 0x72, 0x22, 0x76, 0x0a, 0x20, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72,
	0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x67, 0x72,
	0x65, 0x73, 0x73, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x12, 0x16, 0x0a, 0x06, 0x70, 0x6c,
	0x75, 0x67, 0x69, 0x6e, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52, 0x06, 0x70, 0x6c, 0x75, 0x67,
	0x69, 0x6e, 0x12, 0x20, 0x0a, 0x0b, 0x6f, 0x70, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x49,
	0x44, 0x18, 0x02, 0x20, 0x01, 0x28, 0x09, 0x52, 0x0b, 0x6f, 0x70, 0x65, 0x72, 0x61, 0x74, 0x69,
	0x6f, 0x6e, 0x49, 0x44, 0x12, 0x18, 0x0a, 0x07, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x18,
	0x03, 0x20, 0x01, 0x28, 0x0c, 0x52, 0x07, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x22, 0x5d,
	0x0a, 0x21, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74,
	0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x67, 0x72, 0x65, 0x73, 0x73, 0x52, 0x65, 0x73, 0x70, 0x6f,
	0x6e, 0x73, 0x65, 0x12, 0x38, 0x0a, 0x08, 0x70, 0x72, 0x6f, 0x67, 0x72, 0x65, 0x73, 0x73, 0x18,
	0x01, 0x20, 0x01, 0x28, 0x0b, 0x32, 0x1c, 0x2e, 0x67, 0x65, 0x6e, 0x65, 0x72, 0x61, 0x74, 0x65,
	0x64, 0x2e, 0x4f, 0x70, 0x65, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x67, 0x72,
	0x65, 0x73, 0x73, 0x52, 0x08, 0x70, 0x72, 0x6f, 0x67, 0x72, 0x65, 0x73, 0x73, 0x22, 0x74, 0x0a,
	0x1e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69,
	0x6f, 0x6e, 0x43, 0x61, 0x6e, 0x63, 0x65, 0x6c, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x12,
	0x16, 0x0a, 0x06, 0x70, 0x6c, 0x75, 0x67, 0x69, 0x6e, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52,
	0x06, 0x70, 0x6c, 0x75, 0x67, 0x69, 0x6e, 0x12, 0x20, 0x0a, 0x0b, 0x6f, 0x70, 0x65, 0x72, 0x61,
	0x74, 0x69, 0x6f, 0x6e, 0x49, 0x44, 0x18, 0x02, 0x20, 0x01, 0x28, 0x09, 0x52, 0x0b, 0x6f, 0x70,
	0x65, 0x72, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x49, 0x44, 0x12, 0x18, 0x0a, 0x07, 0x72, 0x65, 0x73,
	0x74, 0x6f, 0x72, 0x65, 0x18, 0x03, 0x20, 0x01, 0x28, 0x0c, 0x52, 0x07, 0x72, 0x65, 0x73, 0x74,
	0x6f, 0x72, 0x65, 0x22, 0x9f, 0x01, 0x0a, 0x22, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49,
	0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x52, 0x65,
	0x61, 0x64, 0x79, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x12, 0x16, 0x0a, 0x06, 0x70, 0x6c,
	0x75, 0x67, 0x69, 0x6e, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09, 0x52, 0x06, 0x70, 0x6c, 0x75, 0x67,
	0x69, 0x6e, 0x12, 0x18, 0x0a, 0x07, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x18, 0x02, 0x20,
	0x01, 0x28, 0x0c, 0x52, 0x07, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x12, 0x47, 0x0a, 0x0f,
	0x61, 0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x18,
	0x03, 0x20, 0x03, 0x28, 0x0b, 0x32, 0x1d, 0x2e, 0x67, 0x65, 0x6e, 0x65, 0x72, 0x61, 0x74, 0x65,
	0x64, 0x2e, 0x52, 0x65, 0x73, 0x6f, 0x75, 0x72, 0x63, 0x65, 0x49, 0x64, 0x65, 0x6e, 0x74, 0x69,
	0x66, 0x69, 0x65, 0x72, 0x52, 0x0f, 0x61, 0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c,
	0x49, 0x74, 0x65, 0x6d, 0x73, 0x22, 0x3b, 0x0a, 0x23, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65,
	0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x52,
	0x65, 0x61, 0x64, 0x79, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x14, 0x0a, 0x05,
	0x72, 0x65, 0x61, 0x64, 0x79, 0x18, 0x01, 0x20, 0x01, 0x28, 0x08, 0x52, 0x05, 0x72, 0x65, 0x61,
	0x64, 0x79, 0x32, 0xd0, 0x03, 0x0a, 0x11, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74,
	0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x12, 0x5a, 0x0a, 0x09, 0x41, 0x70, 0x70, 0x6c,
	0x69, 0x65, 0x73, 0x54, 0x6f, 0x12, 0x25, 0x2e, 0x76, 0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f,
	0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x41, 0x70, 0x70, 0x6c,
	0x69, 0x65, 0x73, 0x54, 0x6f, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x1a, 0x26, 0x2e, 0x76,
	0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74,
	0x69, 0x6f, 0x6e, 0x41, 0x70, 0x70, 0x6c, 0x69, 0x65, 0x73, 0x54, 0x6f, 0x52, 0x65, 0x73, 0x70,
	0x6f, 0x6e, 0x73, 0x65, 0x12, 0x54, 0x0a, 0x07, 0x45, 0x78, 0x65, 0x63, 0x75, 0x74, 0x65, 0x12,
	0x23, 0x2e, 0x76, 0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d,
	0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x45, 0x78, 0x65, 0x63, 0x75, 0x74, 0x65, 0x52, 0x65, 0x71,
	0x75, 0x65, 0x73, 0x74, 0x1a, 0x24, 0x2e, 0x76, 0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72,
	0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x45, 0x78, 0x65, 0x63, 0x75,
	0x74, 0x65, 0x52, 0x65, 0x73, 0x70, 0x6f, 0x6e, 0x73, 0x65, 0x12, 0x57, 0x0a, 0x08, 0x50, 0x72,
	0x6f, 0x67, 0x72, 0x65, 0x73, 0x73, 0x12, 0x24, 0x2e, 0x76, 0x32, 0x2e, 0x52, 0x65, 0x73, 0x74,
	0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f,
	0x67, 0x72, 0x65, 0x73, 0x73, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x1a, 0x25, 0x2e, 0x76,
	0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74,
	0x69, 0x6f, 0x6e, 0x50, 0x72, 0x6f, 0x67, 0x72, 0x65, 0x73, 0x73, 0x52, 0x65, 0x73, 0x70, 0x6f,
	0x6e, 0x73, 0x65, 0x12, 0x44, 0x0a, 0x06, 0x43, 0x61, 0x6e, 0x63, 0x65, 0x6c, 0x12, 0x22, 0x2e,
	0x76, 0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63,
	0x74, 0x69, 0x6f, 0x6e, 0x43, 0x61, 0x6e, 0x63, 0x65, 0x6c, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73,
	0x74, 0x1a, 0x16, 0x2e, 0x67, 0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f,
	0x62, 0x75, 0x66, 0x2e, 0x45, 0x6d, 0x70, 0x74, 0x79, 0x12, 0x6a, 0x0a, 0x17, 0x41, 0x72, 0x65,
	0x41, 0x64, 0x64, 0x69, 0x74, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x52,
	0x65, 0x61, 0x64, 0x79, 0x12, 0x26, 0x2e, 0x76, 0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72,
	0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x49, 0x74, 0x65, 0x6d, 0x73,
	0x52, 0x65, 0x61, 0x64, 0x79, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x1a, 0x27, 0x2e, 0x76,
	0x32, 0x2e, 0x52, 0x65, 0x73, 0x74, 0x6f, 0x72, 0x65, 0x49, 0x74, 0x65, 0x6d, 0x41, 0x63, 0x74,
	0x69, 0x6f, 0x6e, 0x49, 0x74, 0x65, 0x6d, 0x73, 0x52, 0x65, 0x61, 0x64, 0x79, 0x52, 0x65, 0x73,
	0x70, 0x6f, 0x6e, 0x73, 0x65, 0x42, 0x4a, 0x5a, 0x48, 0x67, 0x69, 0x74, 0x68, 0x75, 0x62, 0x2e,
	0x63, 0x6f, 0x6d, 0x2f, 0x76, 0x6d, 0x77, 0x61, 0x72, 0x65, 0x2d, 0x74, 0x61, 0x6e, 0x7a, 0x75,
	0x2f, 0x76, 0x65, 0x6c, 0x65, 0x72, 0x6f, 0x2f, 0x70, 0x6b, 0x67, 0x2f, 0x70, 0x6c, 0x75, 0x67,
	0x69, 0x6e, 0x2f, 0x67, 0x65, 0x6e, 0x65, 0x72, 0x61, 0x74, 0x65, 0x64, 0x2f, 0x72, 0x65, 0x73,
	0x74, 0x6f, 0x72, 0x65, 0x69, 0x74, 0x65, 0x6d, 0x61, 0x63, 0x74, 0x69, 0x6f, 0x6e, 0x2f, 0x76,
	0x32, 0x62, 0x06, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x33,
}

var (
	file_restoreitemaction_v2_RestoreItemAction_proto_rawDescOnce sync.Once
	file_restoreitemaction_v2_RestoreItemAction_proto_rawDescData = file_restoreitemaction_v2_RestoreItemAction_proto_rawDesc
)

func file_restoreitemaction_v2_RestoreItemAction_proto_rawDescGZIP() []byte {
	file_restoreitemaction_v2_RestoreItemAction_proto_rawDescOnce.Do(func() {
		file_restoreitemaction_v2_RestoreItemAction_proto_rawDescData = protoimpl.X.CompressGZIP(file_restoreitemaction_v2_RestoreItemAction_proto_rawDescData)
	})
	return file_restoreitemaction_v2_RestoreItemAction_proto_rawDescData
}

var file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes = make([]protoimpl.MessageInfo, 9)
var file_restoreitemaction_v2_RestoreItemAction_proto_goTypes = []interface{}{
	(*RestoreItemActionExecuteRequest)(nil),     // 0: v2.RestoreItemActionExecuteRequest
	(*RestoreItemActionExecuteResponse)(nil),    // 1: v2.RestoreItemActionExecuteResponse
	(*RestoreItemActionAppliesToRequest)(nil),   // 2: v2.RestoreItemActionAppliesToRequest
	(*RestoreItemActionAppliesToResponse)(nil),  // 3: v2.RestoreItemActionAppliesToResponse
	(*RestoreItemActionProgressRequest)(nil),    // 4: v2.RestoreItemActionProgressRequest
	(*RestoreItemActionProgressResponse)(nil),   // 5: v2.RestoreItemActionProgressResponse
	(*RestoreItemActionCancelRequest)(nil),      // 6: v2.RestoreItemActionCancelRequest
	(*RestoreItemActionItemsReadyRequest)(nil),  // 7: v2.RestoreItemActionItemsReadyRequest
	(*RestoreItemActionItemsReadyResponse)(nil), // 8: v2.RestoreItemActionItemsReadyResponse
	(*generated.ResourceIdentifier)(nil),        // 9: generated.ResourceIdentifier
	(*durationpb.Duration)(nil),                 // 10: google.protobuf.Duration
	(*generated.ResourceSelector)(nil),          // 11: generated.ResourceSelector
	(*generated.OperationProgress)(nil),         // 12: generated.OperationProgress
	(*emptypb.Empty)(nil),                       // 13: google.protobuf.Empty
}
var file_restoreitemaction_v2_RestoreItemAction_proto_depIdxs = []int32{
	9,  // 0: v2.RestoreItemActionExecuteResponse.additionalItems:type_name -> generated.ResourceIdentifier
	10, // 1: v2.RestoreItemActionExecuteResponse.additionalItemsReadyTimeout:type_name -> google.protobuf.Duration
	11, // 2: v2.RestoreItemActionAppliesToResponse.ResourceSelector:type_name -> generated.ResourceSelector
	12, // 3: v2.RestoreItemActionProgressResponse.progress:type_name -> generated.OperationProgress
	9,  // 4: v2.RestoreItemActionItemsReadyRequest.additionalItems:type_name -> generated.ResourceIdentifier
	2,  // 5: v2.RestoreItemAction.AppliesTo:input_type -> v2.RestoreItemActionAppliesToRequest
	0,  // 6: v2.RestoreItemAction.Execute:input_type -> v2.RestoreItemActionExecuteRequest
	4,  // 7: v2.RestoreItemAction.Progress:input_type -> v2.RestoreItemActionProgressRequest
	6,  // 8: v2.RestoreItemAction.Cancel:input_type -> v2.RestoreItemActionCancelRequest
	7,  // 9: v2.RestoreItemAction.AreAdditionalItemsReady:input_type -> v2.RestoreItemActionItemsReadyRequest
	3,  // 10: v2.RestoreItemAction.AppliesTo:output_type -> v2.RestoreItemActionAppliesToResponse
	1,  // 11: v2.RestoreItemAction.Execute:output_type -> v2.RestoreItemActionExecuteResponse
	5,  // 12: v2.RestoreItemAction.Progress:output_type -> v2.RestoreItemActionProgressResponse
	13, // 13: v2.RestoreItemAction.Cancel:output_type -> google.protobuf.Empty
	8,  // 14: v2.RestoreItemAction.AreAdditionalItemsReady:output_type -> v2.RestoreItemActionItemsReadyResponse
	10, // [10:15] is the sub-list for method output_type
	5,  // [5:10] is the sub-list for method input_type
	5,  // [5:5] is the sub-list for extension type_name
	5,  // [5:5] is the sub-list for extension extendee
	0,  // [0:5] is the sub-list for field type_name
}

func init() { file_restoreitemaction_v2_RestoreItemAction_proto_init() }
func file_restoreitemaction_v2_RestoreItemAction_proto_init() {
	if File_restoreitemaction_v2_RestoreItemAction_proto != nil {
		return
	}
	if !protoimpl.UnsafeEnabled {
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[0].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionExecuteRequest); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[1].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionExecuteResponse); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[2].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionAppliesToRequest); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[3].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionAppliesToResponse); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[4].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionProgressRequest); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[5].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionProgressResponse); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[6].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionCancelRequest); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[7].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionItemsReadyRequest); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
		file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes[8].Exporter = func(v interface{}, i int) interface{} {
			switch v := v.(*RestoreItemActionItemsReadyResponse); i {
			case 0:
				return &v.state
			case 1:
				return &v.sizeCache
			case 2:
				return &v.unknownFields
			default:
				return nil
			}
		}
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_restoreitemaction_v2_RestoreItemAction_proto_rawDesc,
			NumEnums:      0,
			NumMessages:   9,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_restoreitemaction_v2_RestoreItemAction_proto_goTypes,
		DependencyIndexes: file_restoreitemaction_v2_RestoreItemAction_proto_depIdxs,
		MessageInfos:      file_restoreitemaction_v2_RestoreItemAction_proto_msgTypes,
	}.Build()
	File_restoreitemaction_v2_RestoreItemAction_proto = out.File
	file_restoreitemaction_v2_RestoreItemAction_proto_rawDesc = nil
	file_restoreitemaction_v2_RestoreItemAction_proto_goTypes = nil
	file_restoreitemaction_v2_RestoreItemAction_proto_depIdxs = nil
}
